'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// We listen to resize events too, but don't actually
// use the event object so this is typed fine for now
var Perimeter = function (_Component) {
  _inherits(Perimeter, _Component);

  function Perimeter(props) {
    _classCallCheck(this, Perimeter);

    var mapListeners = props.mapListeners;

    // The HTML element used as the perimeter center
    var _this = _possibleConstructorReturn(this, (Perimeter.__proto__ || Object.getPrototypeOf(Perimeter)).call(this, props));

    _initialiseProps.call(_this);

    _this.node = null;
    // The result of calling getBoundingClientRect on this.node
    _this.bounds = null;
    // Initial pageYOffset, required since pages may be scrolled on load
    // and getBoundingClientRect will take that into consideration when
    // calculating the top offset
    _this.initialOffset = 0;
    // Whether the mouse is within the perimeter
    _this.breached = false;
    // If we're still listening for mousemove/resize events
    _this.listening = false;
    // If a mapListeners function is provided, update the instance event
    // handlers to be the returned mapped versions. Common use case would be
    // debounced or throttled listeners.
    if (mapListeners) {
      _this.handleMouseMove = mapListeners(_this.handleMouseMove);
      _this.handleResize = mapListeners(_this.handleResize);
    }
    return _this;
  }

  _createClass(Perimeter, [{
    key: 'componentDidMount',


    /**
     * When the component mounts we calculate the ClientRect
     * for the target node and attach event listeners for:
     *   - `mousemove` for checking perimeter breaches
     *   - `resize` for recalculating ClientRect
     */
    value: function componentDidMount() {
      var handleMouseMove = this.handleMouseMove,
          handleResize = this.handleResize,
          node = this.node;

      if (node) {
        this.bounds = node.getBoundingClientRect();
        this.initialOffset = window.pageYOffset;
        window.addEventListener('mousemove', handleMouseMove);
        window.addEventListener('resize', handleResize);
        this.listening = true;
      }
    }

    /**
     * Remove event listeners on unmount if we are still
     * listening.
     */

  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.listening) {
        this.removeEventListeners();
      }
    }

    /**
     * Removes the `mousemove` and `resize` listeners. May
     * be called on unmount, or after `onBreach` is called
     * if the `once` prop is set to `true`
     */

  }, {
    key: 'removeEventListeners',
    value: function removeEventListeners() {
      window.removeEventListener('mousemove', this.handleMouseMove);
      window.removeEventListener('resize', this.handleResize);
      this.listening = false;
    }

    /**
     * An element's position may change on resize, so we recalculate
     * the ClientRect for the node.
     * @todo debounce
     */


    /**
     * Called on `mousemove`, using `clientX` and `clientY`
     * event properties to determine the position of the cursor.
     * The perimeter is then calculated from the ClientRect and the
     * `padding` prop. If the mouse is within the perimeter,
     * and its not already breached (tracked on `this.breached`)
     * @param {MouseEvent} event mouse event
     */


    /**
     * Ref callback used to populate this.node. If
     * a render callback is provided then it will be passed
     * to that callback with the expectation of it then
     * being passed to an element via the `ref` prop somewhere.
     * Otherwise the default `span` is rendered and populated.
     */

  }, {
    key: 'render',
    value: function render() {
      var children = this.props.children;

      return typeof children === 'function' ? children(this.attachRef) : _react2.default.createElement(
        'span',
        { ref: this.attachRef },
        children
      );
    }
  }]);

  return Perimeter;
}(_react.Component);

Perimeter.propTypes = {
  onBreach: _react2.default.PropTypes.func.isRequired,
  once: _react2.default.PropTypes.bool,
  padding: _react2.default.PropTypes.number.isRequired,
  mapListeners: _react2.default.PropTypes.func
};

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.handleResize = function () {
    if (_this2.node) {
      _this2.bounds = _this2.node.getBoundingClientRect();
    }
  };

  this.handleMouseMove = function (_ref) {
    var clientX = _ref.clientX,
        clientY = _ref.clientY;
    var initialOffset = _this2.initialOffset,
        props = _this2.props,
        bounds = _this2.bounds;
    /**
     * There should be no situation where the `mousemove` handler
     * is called and `bounds` is not calculated, but flow demands
     * this check since the initial property value is `null` 
     */

    if (!bounds) return;
    var offsetY = window.pageYOffset - initialOffset;
    var padding = props.padding,
        onBreach = props.onBreach,
        once = props.once;
    var top = bounds.top,
        right = bounds.right,
        bottom = bounds.bottom,
        left = bounds.left;

    if (
    // Cursor is not too far left
    clientX > left - padding &&
    // Cursor is not too far right
    clientX < right + padding &&
    // Cursor is not too far up
    clientY > top - offsetY - padding &&
    // Cursor is not too far down
    clientY < bottom - offsetY + padding) {
      if (_this2.breached) {
        return;
      }
      onBreach();
      _this2.breached = true;
      if (once) {
        _this2.removeEventListeners();
      }
    } else {
      _this2.breached = false;
    }
  };

  this.attachRef = function (node) {
    _this2.node = node;
  };
};

exports.default = Perimeter;